#!/bin/bash
## A script to update the CemTrakker devices.
## This script installs new firmware to the Cemtrakker from an USB-Stick.
## The script ist started when the "Update Firmware" button in pressed
## in the USB-menu.
## Old versions of the Firmware start a executable named "magsus", newer
## versions start "cemtrakker". In future we will call it "ctupdate.sh".


# In the case that this script was started from ~/.profile we wait
# for a few seconds to let that script terminate. Also the usb-stick needs to
# be mounted.
sleep 5

## Ensure that this file is not blocking itself. Since
## it copys a executable to ~/bin/cemtrakker it may not run from
## this path. So first step is to move the script itself to the new
## standard location ~/ctupdate.sh.
BASE=$(basename "$0")
if [ "${BASE}" != "ctupdate.sh" ] ; then
  logger -s "Update CemTrakker Firmware: Must copy ${0} to ctupdate.sh"
  cp $0 ${HOME}/ctupdate.sh
  chmod 750 ${HOME}/ctupdate.sh
  exec ${HOME}/ctupdate.sh
else
  logger -s "Update CemTrakker Firmware: Running..."
fi

## All other executable reside here.
BINDIR="/home/pi/bin"

# A flag if something is updated and we need to reboot at the end.
UPDATED=0

## Escape all slashes in a string by backslashes. Used when handling
## paths by sed
## @param[in] $1 The string to be escaped
function escape()
{
  echo "$1" | sed 's/\//\\\//g'
}

## In some elder implementations the daemon rpigpiod resides in /usr/local/bin.
## We want to move this to /home/pi/bin
function move_rpigpiod()
{
  local GIOD="/usr/local/bin/rpigpiod"
  # Move the executable if it exists in /usr/local/bin
  if [ -x ${GIOD} ] ; then
    logger -s " - Moving rpigpiod to ${BINDIR}"
    sudo mv ${GIOD} ${BINDIR}
    sudo chown pi:pi ${BINDIR}/rpigpiod
    # Edit /etc/rc.local to start the daemon from the correct place
    local SRC="$(escape ${GIOD})"
    local DST="$(escape ${BINDIR})\/rpigpiod"
    sudo sed -i".bak" "s/${SRC}/${DST}/g" /etc/rc.local
  else
    logger -s " - rpigpiod is already at the correct place"
  fi
}

# Copy new GPIO daemon rpigpiod, if exists
function update_rpigpiod()
{
  local SRCDIR=$1
  if [ -f ${SRCDIR}/rpigpiod ] ; then
    sudo killall rpigpiod
    logger -s " - Installing new rpigpiod"
    cp ${SRCDIR}/rpigpiod ${BINDIR}
    chmod 750 ${BINDIR}/rpigpiod
    UPDATED=1
  else
    logger -s " - No new rpigpiod executable"
  fi
}

# Copy new firmware cemtrakker, if exists
function update_cemtrakker()
{
  local SRCDIR=$1
  if [ -f ${SRCDIR}/cemtrakker.bin ] ; then
    killall cemtrakker
    logger -s " - Installing new cemtrakker"
    cp ${SRCDIR}/cemtrakker.bin ${BINDIR}/cemtrakker
    chmod 750 ${BINDIR}/cemtrakker
    UPDATED=1
  elif [ -f ${SRCDIR}/cemtrakker ] ; then
    killall cemtrakker
    logger -s " - Installing new cemtrakker"
    cp ${SRCDIR}/cemtrakker ${BINDIR}
    chmod 750 ${BINDIR}/cemtrakker
    UPDATED=1
  else
    logger -s " - No new cemtrakker executable"
  fi
}

# Install new icons if a archive exists
function update_icons()
{
  local SRCDIR=$1
  local DSTDIR="/usr/local/cemtrakker"
  if [ -f ${SRCDIR}/icons.zip ] ; then
    logger -s " - Installing new icons"
    sudo mkdir -p ${DSTDIR}
    sudo unzip -o -d ${DSTDIR} ${SRCDIR}/icons.zip
    UPDATED=1
  else
    logger -s " - No new icons found"
  fi
}

## Install additional software packages
function install_packages()
{
    local SRCDIR=$1
    local PKGDIR=/home/pi/packages
    if [ -f ${SRCDIR}/packages.zip ] ; then
	logger -s " - Installing additional software packages"
	mkdir -p ${PKGDIR}
	unzip -o -d ${PKGDIR} ${SRCDIR}/packages.zip
	chmod 750 ${PKGDIR}/install.sh
	(cd ${PKGDIR} && ./install.sh)
	rm -rf ${PKGDIR}
    else
	logger -s " - No additional software packages found"
    fi
}

## Elder devices start magsus instead of cemtrakker in .profile. Change this.
function fix_profile()
{
  if [ -x ${BINDIR}/cemtrakker ] ; then
    logger -s " - fixing ~/.profile."
    sed -i".bak" "s/magsus/cemtrakker/g" ${HOME}/.profile
  else
    logger -s " - No executable cemtrakker, leaving profile unchanged."
  fi
}


## Put everything together

mkdir -p ${BINDIR}
move_rpigpiod

SD="/home/pi/update"
if [ -d "${SD}" ] ; then
  update_rpigpiod ${SD}
  update_cemtrakker ${SD}
  update_icons ${SD}
  install_packages ${SD}
  rm -rf ${SD}
fi

if [ ${UPDATED} -lt 1 ] ; then
  SRCDIR=()
  for SD in /media/* ; do
    SRCDIR+=$SD
  done

  if [ ${#SRCDIR[@]} -lt 1 ] ; then
    logger -s " - No stick, not updating"
    exit 0
  fi

  for SD in $SRCDIR ; do
    update_rpigpiod ${SD}
    update_cemtrakker ${SD}
    update_icons ${SD}
    install_packages ${SD}
  done
fi

fix_profile

if [ ${UPDATED} -gt 0 ] ; then
  sudo shutdown -r now
fi

